#!/usr/bin/python3

RETURN = '''# '''

from subprocess import Popen, PIPE, STDOUT


def main():
    module = AnsibleModule(
        argument_spec=dict(
            mode=dict(required=False, default='check'),
            executable=dict(required=False, default='pip'),
            lsb_release=dict(required=False, default='16'),
        ),
        supports_check_mode=False
    )
    constraints = {
        'dummy': 'v.e.r',
        'docutils': '0.15.2',# botocore requires <16
        'chardet': '3.0.4',# Certbot fails with 4.0.0
        'idna': '2.10',# Certbot fails with 3.0
        'ansible': '2.9.15',
        'python-debian': '0.1.40',
        'elasticsearch': '7.17.0',
        'borgbackup': '1.1.17',
        'jinja2': '3.0.3',
        'charset-normalizer': '2.0.12',# Certbot fails with 2.1.0
        'urllib3': '1.26.15',# Certbot fails with 2.0.0
        'blinker': '1.4',# Ubuntu package
        'entrypoints': '0.3',# Ubuntu package
        'pyparted': '3.11.2',
        'pick': '1.2.0'
    }
    mode = module.params['mode']
    executable = module.params['executable']
    lsb_release = module.params['lsb_release']
    if lsb_release == '16':
        constraints['borgmatic'] = '1.5.12'
        constraints['certifi'] = '2021.10.8'
        constraints['requests'] = '2.15.1'
        constraints['ruamel.yaml'] = '0.17.21'
        constraints['setuptools'] = '49.0.0'
    result = []

    changed = (lsb_release == '16') or (lsb_release == '18') or (lsb_release == '20') or (lsb_release == '22')
    if changed:
        process = Popen([executable, 'list', '--disable-pip-version-check', '--outdated', '--format=json'], stdout=PIPE, stderr=STDOUT)
        process.wait()
        updates = []
        for line in process.stdout.read().decode('utf8').split("\n"):
            if line != '' and line[0] == '[':
                updates = json.loads(line)
        for item in updates:
            name = item['name'].lower()
            if name in constraints and constraints[name] == item['version']:
                # This doesn't need an update
                pass
            else:
                result.append(item)

    if len(result) == 0:
        module.exit_json(changed=False, result="No updates available")

    if changed and mode == 'upgrade':
        args = [executable, 'install', '--upgrade']
        if len(constraints) > 0:
            args.append('--constraint=/tmp/ansible-pip-constraints')
            with open('/tmp/ansible-pip-constraints', "w") as file:
                for constraint in constraints:
                    file.write("%s==%s\n" % (constraint, constraints[constraint]))
        for item in result:
            args.append(item['name'])
        process = Popen(args, stdout=PIPE, stderr=PIPE)
        process.wait()
        msg = "%s\n\n%s" % (process.stdout.read().decode('utf8'), process.stderr.read().decode('utf8'))
    else:
        msg = "Available updates:\n"
        for item in result:
            msg += "%s %s (installed: %s)\n" % (item['name'], item['latest_version'], item['version'])

    module.exit_json(changed=changed, result=msg)


from ansible.module_utils.basic import *

if __name__ == '__main__':
    main()
